@@ -18,6 +18,12 @@ from .np_deterministic import (
     _spearman_r_eff_p_value,
     _spearman_r_p_value,
 )
+from .utils import (
+    _fail_if_dim_empty,
+    _preprocess_dims,
+    _preprocess_weights,
+    _stack_input_if_needed,
+)
 
 __all__ = [
     'pearson_r',
@@ -37,100 +43,6 @@ __all__ = [
 ]
 
 
-def _preprocess_dims(dim):
-    """Preprocesses dimensions to prep for stacking.
-
-    Parameters
-    ----------
-    dim : str, list
-        The dimension(s) to apply the function along.
-    """
-    if isinstance(dim, str):
-        dim = [dim]
-    axis = tuple(range(-1, -len(dim) - 1, -1))
-    return dim, axis
-
-
-def _stack_input_if_needed(a, b, dim, weights):
-    """
-    Stack input arrays a, b if needed in correlation metrics.
-    Adapt dim and weights accordingly.
-
-    Parameters
-    ----------
-    a : xarray.Dataset or xarray.DataArray
-        Labeled array(s) over which to apply the function.
-    b : xarray.Dataset or xarray.DataArray
-        Labeled array(s) over which to apply the function.
-    dim : list
-        The dimension(s) to apply the correlation along.
-    weights : xarray.Dataset or xarray.DataArray or None
-        Weights matching dimensions of ``dim`` to apply during the function.
-
-    Returns
-    -------
-    a : xarray.Dataset or xarray.DataArray stacked with new_dim
-        Labeled array(s) over which to apply the function.
-    b : xarray.Dataset or xarray.DataArray stacked with new_dim
-        Labeled array(s) over which to apply the function.
-    new_dim : str
-        The dimension(s) to apply the correlation along.
-    weights : xarray.Dataset or xarray.DataArray stacked with new_dim or None
-        Weights matching dimensions of ``dim`` to apply during the function.
-
-    """
-    if len(dim) > 1:
-        new_dim = '_'.join(dim)
-        a = a.stack(**{new_dim: dim})
-        b = b.stack(**{new_dim: dim})
-        if weights is not None:
-            weights = weights.stack(**{new_dim: dim})
-    else:
-        new_dim = dim[0]
-    return a, b, new_dim, weights
-
-
-def _preprocess_weights(a, dim, new_dim, weights):
-    """Preprocesses weights array to prepare for numpy computation.
-
-    Parameters
-    ----------
-    a : xarray.Dataset or xarray.DataArray
-        One of the arrays over which the function will be applied.
-    dim : str, list
-        The original dimension(s) to apply the function along.
-    new_dim : str
-        The newly named dimension after running ``_preprocess_dims``
-    weights : xarray.Dataset or xarray.DataArray or None
-        Weights to apply to function, matching the dimension size of
-        ``new_dim``.
-    """
-    if weights is None:
-        return None
-    else:
-        # Throw error if there are negative weights.
-        if weights.min() < 0:
-            raise ValueError(
-                'Weights has a minimum below 0. Please submit a weights array '
-                'of positive numbers.'
-            )
-        # Scale weights to vary from 0 to 1.
-        weights = weights / weights.max()
-        # Check that the weights array has the same size
-        # dimension(s) as those being applied over.
-        drop_dims = {k: 0 for k in a.dims if k not in new_dim}
-        if dict(weights.sizes) != dict(a.isel(drop_dims).sizes):
-            raise ValueError(
-                f'weights dimension(s) {dim} of size {dict(weights.sizes)} '
-                f"does not match DataArray's size "
-                f'{dict(a.isel(drop_dims).sizes)}'
-            )
-        if dict(weights.sizes) != dict(a.sizes):
-            # Broadcast weights to full size of main object.
-            _, weights = xr.broadcast(a, weights)
-        return weights
-
-
 def _determine_input_core_dims(dim, weights):
     """
     Determine input_core_dims based on type of dim and weights.
@@ -138,7 +50,7 @@ def _determine_input_core_dims(dim, weights):
     Parameters
     ----------
     dim : str, list
-        The dimension(s) to apply the correlation along.
+        The dimension(s) to apply the metric along.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
 
@@ -157,9 +69,13 @@ def _determine_input_core_dims(dim, weights):
     return input_core_dims
 
 
-def pearson_r(a, b, dim, weights=None, skipna=False):
-    """
-    Pearson's correlation coefficient.
+def pearson_r(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """Pearson's correlation coefficient.
+
+    .. math::
+        r_{ab} = \\frac{ \\sum_{i=i}^{n} (a_{i} - \\bar{a}) (b_{i} - \\bar{b}) }
+                 {\\sqrt{ \\sum_{i=1}^{n} (a_{i} - \\bar{a})^{2} }
+                  \\sqrt{ \\sum_{i=1}^{n} (b_{i} - \\bar{b})^{2} }}
 
     Parameters
     ----------
@@ -168,11 +84,17 @@ def pearson_r(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the correlation along.
+        The dimension(s) to apply the correlation along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -181,16 +103,26 @@ def pearson_r(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     scipy.stats.pearsonr
-    xskillscore.core.np_deterministic._pearson_r
 
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Pearson_correlation_coefficient
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import pearson_r
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> pearson_r(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     a, b, new_dim, weights = _stack_input_if_needed(a, b, dim, weights)
     weights = _preprocess_weights(a, dim, new_dim, weights)
 
@@ -205,12 +137,35 @@ def pearson_r(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def r2(a, b, dim, weights=None, skipna=False):
-    """
-    R^2 (coefficient of determination) score.
+def r2(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """R^2 (coefficient of determination) score.
+
+    We first take the total sum of squares of our known vector, a.
+
+    .. math::
+        SS_{\\mathrm{tot}} = \\sum_{i=1}^{n} (a_{i} - \\bar{a})^{2}
+
+    Next, we take the sum of squares of the error between our known vector
+    a and the predicted vector, b.
+
+    .. math::
+        SS_{\\mathrm{res}} = \\sum_{i=1}^{n} (a_{i} - b_{i})^{2}
+
+    Lastly we compute the coefficient of determiniation using these two
+    terms.
+
+    .. math::
+        R^{2} = 1 - \\frac{SS_{\\mathrm{res}}}{SS_{\\mathrm{tot}}}
+
+    .. note::
+        The coefficient of determination is *not* symmetric. In other words,
+        ``r2(a, b) != r2(b, a)``. Be careful and note that by our
+        convention, ``b`` is the modeled/predicted vector and ``a`` is the
+        observed vector.
 
     Parameters
     ----------
@@ -219,11 +174,17 @@ def r2(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the correlation along.
+        The dimension(s) to apply the correlation along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -232,14 +193,26 @@ def r2(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     sklearn.metrics.r2_score
 
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Coefficient_of_determination
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import r2
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> r2(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     a, b, new_dim, weights = _stack_input_if_needed(a, b, dim, weights)
     weights = _preprocess_weights(a, dim, new_dim, weights)
 
@@ -254,12 +227,12 @@ def r2(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def pearson_r_p_value(a, b, dim, weights=None, skipna=False):
-    """
-    2-tailed p-value associated with pearson's correlation coefficient.
+def pearson_r_p_value(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """2-tailed p-value associated with pearson's correlation coefficient.
 
     Parameters
     ----------
@@ -268,11 +241,17 @@ def pearson_r_p_value(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the correlation along.
+        The dimension(s) to apply the correlation along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -281,12 +260,22 @@ def pearson_r_p_value(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     scipy.stats.pearsonr
-    xskillscore.core.np_deterministic._pearson_r_p_value
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import pearson_r_p_value
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> pearson_r_p_value(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     a, b, new_dim, weights = _stack_input_if_needed(a, b, dim, weights)
     weights = _preprocess_weights(a, dim, new_dim, weights)
     input_core_dims = _determine_input_core_dims(new_dim, weights)
@@ -300,10 +289,11 @@ def pearson_r_p_value(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def effective_sample_size(a, b, dim, skipna=False):
+def effective_sample_size(a, b, dim='time', skipna=False, keep_attrs=False):
     """Effective sample size for temporally correlated data.
 
     .. note::
@@ -332,24 +322,43 @@ def effective_sample_size(a, b, dim, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the function along.
+        The dimension(s) to apply the function along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
     xarray.Dataset or xarray.DataArray
         Effective sample size.
 
-    Reference
-    ---------
+    References
+    ----------
     * Bretherton, Christopher S., et al. "The effective number of spatial degrees of
       freedom of a time-varying field." Journal of climate 12.7 (1999): 1990-2009.
     * Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
       Academic press, 2011.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import effective_sample_size
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> effective_sample_size(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
+
     if len(dim) > 1:
         raise ValueError(
             'Effective sample size should only be applied to a singular time dimension.'
@@ -370,10 +379,11 @@ def effective_sample_size(a, b, dim, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def pearson_r_eff_p_value(a, b, dim, skipna=False):
+def pearson_r_eff_p_value(a, b, dim=None, skipna=False, keep_attrs=False):
     """
     2-tailed p-value associated with Pearson's correlation coefficient,
     accounting for autocorrelation.
@@ -409,9 +419,15 @@ def pearson_r_eff_p_value(a, b, dim, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to compute the p value over.
+        The dimension(s) to compute the p value over. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -421,19 +437,30 @@ def pearson_r_eff_p_value(a, b, dim, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     scipy.stats.pearsonr
-    xskillscore.core.np_deterministic._pearson_r_eff_p_value
 
-    Reference
-    ---------
+    References
+    ----------
     * Bretherton, Christopher S., et al. "The effective number of spatial degrees of
       freedom of a time-varying field." Journal of climate 12.7 (1999): 1990-2009.
     * Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
       Academic press, 2011.
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import pearson_r_eff_p_value
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> pearson_r_eff_p_value(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
+
     if len(dim) > 1:
         raise ValueError(
             'Effective sample size should only be applied to a singular time dimension.'
@@ -454,12 +481,12 @@ def pearson_r_eff_p_value(a, b, dim, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def spearman_r(a, b, dim, weights=None, skipna=False):
-    """
-    Spearman's correlation coefficient.
+def spearman_r(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """Spearman's correlation coefficient.
 
     Parameters
     ----------
@@ -468,11 +495,17 @@ def spearman_r(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the correlation along.
+        The dimension(s) to apply the correlation along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -481,17 +514,27 @@ def spearman_r(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     scipy.stats.spearman_r
-    xskillscore.core.np_deterministic._spearman_r
 
-    Reference
-    ---------
+    References
+    ----------
     https://github.com/scipy/scipy/blob/v1.3.1/scipy/stats/stats.py#L3613-L3764
     https://en.wikipedia.org/wiki/Spearman%27s_rank_correlation_coefficient
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import spearman_r
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> spearman_r(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     a, b, new_dim, weights = _stack_input_if_needed(a, b, dim, weights)
     weights = _preprocess_weights(a, dim, new_dim, weights)
     input_core_dims = _determine_input_core_dims(new_dim, weights)
@@ -505,12 +548,12 @@ def spearman_r(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def spearman_r_p_value(a, b, dim, weights=None, skipna=False):
-    """
-    2-tailed p-value associated with Spearman's correlation coefficient.
+def spearman_r_p_value(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """2-tailed p-value associated with Spearman's correlation coefficient.
 
     Parameters
     ----------
@@ -519,11 +562,17 @@ def spearman_r_p_value(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the correlation along.
+        The dimension(s) to apply the correlation along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -532,12 +581,22 @@ def spearman_r_p_value(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     scipy.stats.spearman_r
-    xskillscore.core.np_deterministic._spearman_r_p_value
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import spearman_r_p_value
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> spearman_r_p_value(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     a, b, new_dim, weights = _stack_input_if_needed(a, b, dim, weights)
     weights = _preprocess_weights(a, dim, new_dim, weights)
     input_core_dims = _determine_input_core_dims(new_dim, weights)
@@ -551,10 +610,11 @@ def spearman_r_p_value(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def spearman_r_eff_p_value(a, b, dim, skipna=False):
+def spearman_r_eff_p_value(a, b, dim=None, skipna=False, keep_attrs=False):
     """
     2-tailed p-value associated with Spearman rank correlation coefficient,
     accounting for autocorrelation.
@@ -590,9 +650,15 @@ def spearman_r_eff_p_value(a, b, dim, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to compute the p value over.
+        The dimension(s) to compute the p value over. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -600,21 +666,34 @@ def spearman_r_eff_p_value(a, b, dim, skipna=False):
         2-tailed p-value of Spearman's correlation coefficient, accounting for
         autocorrelation.
 
-    Reference
-    ---------
+    See Also
+    --------
+    scipy.stats.spearman_r
+
+    References
+    ----------
     * Bretherton, Christopher S., et al. "The effective number of spatial degrees of
       freedom of a time-varying field." Journal of climate 12.7 (1999): 1990-2009.
     * Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
       Academic press, 2011.
 
-    See Also
-    --------
-    xarray.apply_ufunc
-    scipy.stats.spearman_r
-    xskillscore.core.np_deterministic._spearman_r_eff_p_value
 
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import spearman_r_eff_p_value
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> spearman_r_eff_p_value(a, b, dim='time')
     """
-    dim, _ = _preprocess_dims(dim)
+    _fail_if_dim_empty(dim)
+    dim, _ = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
+
     if len(dim) > 1:
         raise ValueError(
             'Effective sample size should only be applied to a singular time dimension.'
@@ -635,12 +714,15 @@ def spearman_r_eff_p_value(a, b, dim, skipna=False):
         kwargs={'axis': -1, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def rmse(a, b, dim, weights=None, skipna=False):
-    """
-    Root Mean Squared Error.
+def rmse(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """Root Mean Squared Error.
+
+    .. math::
+        \\mathrm{RMSE} = \\sqrt{\\frac{1}{n}\\sum_{i=1}^{n}(a_{i} - b_{i})^{2}}
 
     Parameters
     ----------
@@ -649,11 +731,17 @@ def rmse(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the rmse along.
+        The dimension(s) to apply the rmse along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -662,16 +750,25 @@ def rmse(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     sklearn.metrics.mean_squared_error
-    xskillscore.core.np_deterministic._rmse
 
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Root-mean-square_deviation
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import rmse
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> rmse(a, b, dim='time')
     """
-    dim, axis = _preprocess_dims(dim)
+    dim, axis = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     weights = _preprocess_weights(a, dim, dim, weights)
     input_core_dims = _determine_input_core_dims(dim, weights)
 
@@ -684,12 +781,15 @@ def rmse(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': axis, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def mse(a, b, dim, weights=None, skipna=False):
-    """
-    Mean Squared Error.
+def mse(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """Mean Squared Error.
+
+    .. math::
+        \\mathrm{MSE} = \\frac{1}{n}\\sum_{i=1}^{n}(a_{i} - b_{i})^{2}
 
     Parameters
     ----------
@@ -698,11 +798,17 @@ def mse(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the mse along.
+        The dimension(s) to apply the mse along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -711,16 +817,25 @@ def mse(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     sklearn.metrics.mean_squared_error
-    xskillscore.core.np_deterministic._mse
 
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Mean_squared_error
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import mse
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> mse(a, b, dim='time')
     """
-    dim, axis = _preprocess_dims(dim)
+    dim, axis = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     weights = _preprocess_weights(a, dim, dim, weights)
     input_core_dims = _determine_input_core_dims(dim, weights)
 
@@ -733,12 +848,15 @@ def mse(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': axis, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def mae(a, b, dim, weights=None, skipna=False):
-    """
-    Mean Absolute Error.
+def mae(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """Mean Absolute Error.
+
+    .. math::
+        \\mathrm{MAE} = \\frac{1}{n}\\sum_{i=1}^{n}\\vert a - b\\vert
 
     Parameters
     ----------
@@ -747,11 +865,17 @@ def mae(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the mae along.
+        The dimension(s) to apply the mae along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -760,16 +884,25 @@ def mae(a, b, dim, weights=None, skipna=False):
 
     See Also
     --------
-    xarray.apply_ufunc
     sklearn.metrics.mean_absolute_error
-    xskillscore.core.np_deterministic._mae
 
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Mean_absolute_error
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import mae
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> mae(a, b, dim='time')
     """
-    dim, axis = _preprocess_dims(dim)
+    dim, axis = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     weights = _preprocess_weights(a, dim, dim, weights)
     input_core_dims = _determine_input_core_dims(dim, weights)
 
@@ -782,13 +915,17 @@ def mae(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': axis, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def median_absolute_error(a, b, dim, skipna=False):
+def median_absolute_error(a, b, dim=None, skipna=False, keep_attrs=False):
     """
     Median Absolute Error.
 
+    .. math::
+        \\mathrm{median}(\\vert a - b\\vert)
+
     Parameters
     ----------
     a : xarray.Dataset or xarray.DataArray
@@ -797,8 +934,14 @@ def median_absolute_error(a, b, dim, skipna=False):
         Labeled array(s) over which to apply the function.
     dim : str, list
         The dimension(s) to apply the median absolute error along.
+        Note that this dimension will be reduced as a result. Defaults to None reducing all dimensions.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
@@ -808,11 +951,20 @@ def median_absolute_error(a, b, dim, skipna=False):
     See Also
     --------
     sklearn.metrics.median_absolute_error
-    xarray.apply_ufunc
-    xskillscore.core.np_deterministic._median_absolute_error
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import median_absolute_error
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> median_absolute_error(a, b, dim='time')
     """
-    dim, axis = _preprocess_dims(dim)
+    dim, axis = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
 
     return xr.apply_ufunc(
         _median_absolute_error,
@@ -822,12 +974,22 @@ def median_absolute_error(a, b, dim, skipna=False):
         kwargs={'axis': axis, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def mape(a, b, dim, weights=None, skipna=False):
-    """
-    Mean Absolute Percentage Error.
+def mape(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """Mean Absolute Percentage Error.
+
+    .. math::
+        \\mathrm{MAPE} = \\frac{1}{n} \\sum_{i=1}^{n}
+                         \\frac{\\vert a_{i} - b_{i} \\vert}
+                               {\\vert a_{i} \\vert}
+
+    .. note::
+        The percent error is calculated in reference to ``a``. Percent
+        error is reported as decimal percent. I.e., a value of 1 is
+        100%.
 
     Parameters
     ----------
@@ -837,28 +999,40 @@ def mape(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the mae along.
+        The dimension(s) to apply mape along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
     xarray.Dataset or xarray.DataArray
         Mean Absolute Percentage Error.
 
-    See Also
-    --------
-    xarray.apply_ufunc
-    xskillscore.core.np_deterministic._mape
-
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Mean_absolute_percentage_error
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import mape
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> mape(a, b, dim='time')
     """
-    dim, axis = _preprocess_dims(dim)
+    dim, axis = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     weights = _preprocess_weights(a, dim, dim, weights)
     input_core_dims = _determine_input_core_dims(dim, weights)
 
@@ -871,12 +1045,21 @@ def mape(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': axis, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
 
 
-def smape(a, b, dim, weights=None, skipna=False):
-    """
-    Symmetric Mean Absolute Percentage Error.
+def smape(a, b, dim=None, weights=None, skipna=False, keep_attrs=False):
+    """Symmetric Mean Absolute Percentage Error.
+
+    .. math::
+        \\mathrm{SMAPE} = \\frac{1}{n} \\sum_{i=1}^{n}
+                          \\frac{ \\vert a_{i} - b_{i} \\vert }
+                          { \\vert a_{i} \\vert + \\vert b_{i} \\vert  }
+
+    .. note::
+        Percent error is reported as decimal percent. I.e., a value of 1 is
+        100%.
 
     Parameters
     ----------
@@ -886,28 +1069,40 @@ def smape(a, b, dim, weights=None, skipna=False):
     b : xarray.Dataset or xarray.DataArray
         Labeled array(s) over which to apply the function.
     dim : str, list
-        The dimension(s) to apply the mae along.
+        The dimension(s) to apply the smape along. Note that this dimension will
+        be reduced as a result. Defaults to None reducing all dimensions.
     weights : xarray.Dataset or xarray.DataArray or None
         Weights matching dimensions of ``dim`` to apply during the function.
     skipna : bool
         If True, skip NaNs when computing function.
+    keep_attrs : bool
+        If True, the attributes (attrs) will be copied
+        from the first input to the new one.
+        If False (default), the new object will
+        be returned without attributes.
 
     Returns
     -------
     xarray.Dataset or xarray.DataArray
         Symmetric Mean Absolute Percentage Error.
 
-    See Also
-    --------
-    xarray.apply_ufunc
-    xskillscore.core.np_deterministic._smape
-
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Symmetric_mean_absolute_percentage_error
 
+    Examples
+    --------
+    >>> import numpy as np
+    >>> import xarray as xr
+    >>> from xskillscore import smape
+    >>> a = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> b = xr.DataArray(np.random.rand(5, 3, 3),
+                        dims=['time', 'x', 'y'])
+    >>> smape(a, b, dim='time')
     """
-    dim, axis = _preprocess_dims(dim)
+    dim, axis = _preprocess_dims(dim, a)
+    a, b = xr.broadcast(a, b, exclude=dim)
     weights = _preprocess_weights(a, dim, dim, weights)
     input_core_dims = _determine_input_core_dims(dim, weights)
 
@@ -920,4 +1115,5 @@ def smape(a, b, dim, weights=None, skipna=False):
         kwargs={'axis': axis, 'skipna': skipna},
         dask='parallelized',
         output_dtypes=[float],
+        keep_attrs=keep_attrs,
     )
