@@ -30,6 +30,7 @@ def _match_nans(a, b, weights):
     a, b, weights : ndarray
         a, b, and weights (if not None) with nans placed at
         pairwise locations.
+
     """
     if np.isnan(a).any() or np.isnan(b).any():
         # Avoids mutating original arrays and bypasses read-only issue.
@@ -37,9 +38,11 @@ def _match_nans(a, b, weights):
         # Find pairwise indices in a and b that have nans.
         idx = np.logical_or(np.isnan(a), np.isnan(b))
         a[idx], b[idx] = np.nan, np.nan
-        if weights is not None:
-            weights = weights.copy()
-            weights[idx] = np.nan
+        # https://github.com/raybellwaves/xskillscore/issues/168
+        if isinstance(weights, np.ndarray):
+            if weights.shape:  # not None
+                weights = weights.copy()
+                weights[idx] = np.nan
     return a, b, weights
 
 
@@ -110,13 +113,12 @@ def _effective_sample_size(a, b, axis, skipna):
     n_eff : ndarray
         Effective sample size.
 
-    Reference
-    ---------
-    * Bretherton, Christopher S., et al. "The effective number of spatial degrees of
+    References
+    ----------
+    Bretherton, Christopher S., et al. "The effective number of spatial degrees of
       freedom of a time-varying field." Journal of climate 12.7 (1999): 1990-2009.
-    * Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
+    Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
       Academic press, 2011.
-
     """
     if skipna:
         a, b, _ = _match_nans(a, b, None)
@@ -139,8 +141,7 @@ def _effective_sample_size(a, b, axis, skipna):
 
 
 def _pearson_r(a, b, weights, axis, skipna):
-    """
-    ndarray implementation of scipy.stats.pearsonr.
+    """ndarray implementation of scipy.stats.pearsonr.
 
     Parameters
     ----------
@@ -163,7 +164,6 @@ def _pearson_r(a, b, weights, axis, skipna):
     See Also
     --------
     scipy.stats.pearsonr
-
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
     if skipna:
@@ -191,8 +191,7 @@ def _pearson_r(a, b, weights, axis, skipna):
 
 
 def _r2(a, b, weights, axis, skipna):
-    """
-    ndarray implementation of sklearn.metrics.r2_score.
+    """ndarray implementation of sklearn.metrics.r2_score.
 
     Parameters
     ----------
@@ -216,8 +215,8 @@ def _r2(a, b, weights, axis, skipna):
     --------
     sklearn.metrics.r2_score
 
-    Reference
-    ---------
+    References
+    ----------
     https://en.wikipedia.org/wiki/Coefficient_of_determination
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
@@ -244,8 +243,7 @@ def _r2(a, b, weights, axis, skipna):
 
 
 def _pearson_r_p_value(a, b, weights, axis, skipna):
-    """
-    ndarray implementation of scipy.stats.pearsonr.
+    """ndarray implementation of scipy.stats.pearsonr.
 
     Parameters
     ----------
@@ -268,7 +266,6 @@ def _pearson_r_p_value(a, b, weights, axis, skipna):
     See Also
     --------
     scipy.stats.pearsonr
-
     """
     if skipna:
         a, b, weights = _match_nans(a, b, weights)
@@ -320,13 +317,12 @@ def _pearson_r_eff_p_value(a, b, axis, skipna):
     res : ndarray
         2-tailed p-value.
 
-    Reference
-    ---------
-    * Bretherton, Christopher S., et al. "The effective number of spatial degrees of
+    References
+    ----------
+    Bretherton, Christopher S., et al. "The effective number of spatial degrees of
       freedom of a time-varying field." Journal of climate 12.7 (1999): 1990-2009.
-    * Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
+    Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
       Academic press, 2011.
-
     """
     if skipna:
         a, b, _ = _match_nans(a, b, None)
@@ -350,8 +346,7 @@ def _pearson_r_eff_p_value(a, b, axis, skipna):
 
 
 def _spearman_r(a, b, weights, axis, skipna):
-    """
-    ndarray implementation of scipy.stats.spearmanr.
+    """ndarray implementation of scipy.stats.spearmanr.
 
     Parameters
     ----------
@@ -374,7 +369,6 @@ def _spearman_r(a, b, weights, axis, skipna):
     See Also
     --------
     scipy.stats.spearmanr
-
     """
     if skipna:
         a, b, weights = _match_nans(a, b, weights)
@@ -385,8 +379,7 @@ def _spearman_r(a, b, weights, axis, skipna):
 
 
 def _spearman_r_p_value(a, b, weights, axis, skipna):
-    """
-    ndarray implementation of scipy.stats.spearmanr.
+    """ndarray implementation of scipy.stats.spearmanr.
 
     Parameters
     ----------
@@ -410,10 +403,9 @@ def _spearman_r_p_value(a, b, weights, axis, skipna):
     --------
     scipy.stats.spearmanr
 
-    Reference
-    ---------
+    References
+    ----------
     https://github.com/scipy/scipy/blob/v1.3.1/scipy/stats/stats.py#L3613-L3764
-
     """
     if skipna:
         a, b, weights = _match_nans(a, b, weights)
@@ -456,13 +448,12 @@ def _spearman_r_eff_p_value(a, b, axis, skipna):
     --------
     scipy.stats.spearmanr
 
-    Reference
-    ---------
-    * Bretherton, Christopher S., et al. "The effective number of spatial degrees of
+    References
+    ----------
+    Bretherton, Christopher S., et al. "The effective number of spatial degrees of
       freedom of a time-varying field." Journal of climate 12.7 (1999): 1990-2009.
-    * Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
+    Wilks, Daniel S. Statistical methods in the atmospheric sciences. Vol. 100.
       Academic press, 2011.
-
     """
     if skipna:
         a, b, _ = _match_nans(a, b, None)
@@ -474,8 +465,7 @@ def _spearman_r_eff_p_value(a, b, axis, skipna):
 
 
 def _rmse(a, b, weights, axis, skipna):
-    """
-    Root Mean Squared Error.
+    """Root Mean Squared Error.
 
     Parameters
     ----------
@@ -498,7 +488,6 @@ def _rmse(a, b, weights, axis, skipna):
     See Also
     --------
     sklearn.metrics.mean_squared_error
-
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
     if skipna:
@@ -517,8 +506,7 @@ def _rmse(a, b, weights, axis, skipna):
 
 
 def _mse(a, b, weights, axis, skipna):
-    """
-    Mean Squared Error.
+    """Mean Squared Error.
 
     Parameters
     ----------
@@ -541,7 +529,6 @@ def _mse(a, b, weights, axis, skipna):
     See Also
     --------
     sklearn.metrics.mean_squared_error
-
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
     if skipna:
@@ -556,8 +543,7 @@ def _mse(a, b, weights, axis, skipna):
 
 
 def _mae(a, b, weights, axis, skipna):
-    """
-    Mean Absolute Error.
+    """Mean Absolute Error.
 
     Parameters
     ----------
@@ -580,7 +566,6 @@ def _mae(a, b, weights, axis, skipna):
     See Also
     --------
     sklearn.metrics.mean_absolute_error
-
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
     if skipna:
@@ -597,8 +582,7 @@ def _mae(a, b, weights, axis, skipna):
 
 
 def _median_absolute_error(a, b, axis, skipna):
-    """
-    Median Absolute Error.
+    """Median Absolute Error.
 
     Parameters
     ----------
@@ -619,7 +603,6 @@ def _median_absolute_error(a, b, axis, skipna):
     See Also
     --------
     sklearn.metrics.median_absolute_error
-
     """
     medianfunc = np.nanmedian if skipna else np.median
     if skipna:
@@ -629,10 +612,12 @@ def _median_absolute_error(a, b, axis, skipna):
 
 
 def _mape(a, b, weights, axis, skipna):
-    """
-    Mean Absolute Percentage Error.
+    """Mean Absolute Percentage Error.
 
-    :: math MAPE = 1/n \\sum \\frac{|F_t-A_t|}{|A_t|}
+    .. math::
+        \\mathrm{MAPE} = \\frac{1}{n} \\sum_{i=1}^{n}
+                         \\frac{\\vert a_{i} - b_{i} \\vert}
+                               {\\vert a_{i} \\vert}
 
     Parameters
     ----------
@@ -652,16 +637,16 @@ def _mape(a, b, weights, axis, skipna):
     res : ndarray
         Mean Absolute Percentage Error.
 
-    Reference
-    ---------
-    https://en.wikipedia.org/wiki/Mean_absolute_percentage_error
-
     Notes
     -----
     The percent error is calculated in reference to ``a``.
 
     Percent error is reported as decimal percent. I.e., a value of
     1 is 100%.
+
+    References
+    ----------
+    https://en.wikipedia.org/wiki/Mean_absolute_percentage_error
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
     if skipna:
@@ -676,10 +661,12 @@ def _mape(a, b, weights, axis, skipna):
 
 
 def _smape(a, b, weights, axis, skipna):
-    """
-    Symmetric Mean Absolute Percentage Error.
+    """Symmetric Mean Absolute Percentage Error.
 
-    :: math SMAPE = 1/n \\sum \\frac{|F_t-A_t|}{(|A_t|+|F_t|)}
+    .. math::
+        \\mathrm{SMAPE} = \\frac{1}{n} \\sum_{i=1}^{n}
+                          \\frac{ \\vert a_{i} - b_{i} \\vert }
+                          { \\vert a_{i} \\vert + \\vert b_{i} \\vert  }
 
     Parameters
     ----------
@@ -699,14 +686,14 @@ def _smape(a, b, weights, axis, skipna):
     res : ndarray
         Symmetric Mean Absolute Percentage Error.
 
-    Reference
-    ---------
-    https://en.wikipedia.org/wiki/Symmetric_mean_absolute_percentage_error
-
     Notes
     -----
     Symmetric percent error is reported as decimal percent. I.e., a value of 1
     is 100%.
+
+    References
+    ----------
+    https://en.wikipedia.org/wiki/Symmetric_mean_absolute_percentage_error
     """
     sumfunc, meanfunc = _get_numpy_funcs(skipna)
     if skipna:
