@@ -49,43 +49,12 @@ distance_metrics = [
 AXES = ('time', 'lat', 'lon', ['lat', 'lon'], ['time', 'lat', 'lon'])
 
 
-@pytest.fixture
-def a():
-    times = xr.cftime_range('2000-01-01', '2000-01-03', freq='D')
-    lats = np.arange(4)
-    lons = np.arange(5)
-    data = np.random.rand(len(times), len(lats), len(lons))
-    return xr.DataArray(data, coords=[times, lats, lons], dims=['time', 'lat', 'lon'])
-
-
-@pytest.fixture
-def b(a):
-    b = a.copy()
-    b.values = np.random.rand(a.shape[0], a.shape[1], a.shape[2])
-    return b
-
-
-@pytest.fixture
-def weights(a):
-    """Weighting array by cosine of the latitude."""
-    a_weighted = a.copy()
-    cos = np.abs(np.cos(a.lat))
-    data = np.tile(cos, (a.shape[0], a.shape[2], 1)).reshape(
-        a.shape[0], a.shape[1], a.shape[2]
-    )
-    a_weighted.values = data
-    return a_weighted
-
-
-def _ds(a, b, skipna_bool, dask_bool):
+def _ds(a, b, skipna_bool):
     ds = xr.Dataset()
     ds['a'] = a
     ds['b'] = b
     if skipna_bool is True:
         ds['b'] = b.where(b < 0.5)
-    if dask_bool is True:
-        ds['a'] = a.chunk()
-        ds['b'] = b.chunk()
     return ds
 
 
@@ -106,10 +75,9 @@ def adjust_weights(dim, weight_bool, weights):
 @pytest.mark.parametrize('metric', correlation_metrics + distance_metrics)
 @pytest.mark.parametrize('dim', AXES)
 @pytest.mark.parametrize('weight_bool', [False, True])
-@pytest.mark.parametrize('dask_bool', [False, True])
 @pytest.mark.parametrize('skipna_bool', [False, True])
 def test_deterministic_metrics_accessor(
-    a, b, dim, skipna_bool, dask_bool, weight_bool, weights, metric, outer_bool
+    a, b, dim, skipna_bool, weight_bool, weights, metric, outer_bool
 ):
 
     # Update dim to time if testing temporal only metrics
@@ -117,24 +85,26 @@ def test_deterministic_metrics_accessor(
         dim = 'time'
 
     _weights = adjust_weights(dim, weight_bool, weights)
-    ds = _ds(a, b, skipna_bool, dask_bool)
+    ds = _ds(a, b, skipna_bool)
     b = ds['b']  # Update if populated with nans
     if outer_bool:
         ds = ds.drop_vars('b')
 
     accessor_func = getattr(ds.xs, metric.__name__)
     if metric in temporal_only_metrics or metric == median_absolute_error:
-        actual = metric(a, b, dim, skipna=skipna_bool)
+        actual = metric(a, b, dim=dim, skipna=skipna_bool)
         if outer_bool:
-            expected = accessor_func('a', b, dim, skipna=skipna_bool)
+            expected = accessor_func('a', b, dim=dim, skipna=skipna_bool)
         else:
-            expected = accessor_func('a', 'b', dim, skipna=skipna_bool)
+            expected = accessor_func('a', 'b', dim=dim, skipna=skipna_bool)
     else:
-        actual = metric(a, b, dim, weights=_weights, skipna=skipna_bool)
+        actual = metric(a, b, dim=dim, weights=_weights, skipna=skipna_bool)
         if outer_bool:
-            expected = accessor_func('a', b, dim, weights=_weights, skipna=skipna_bool)
+            expected = accessor_func(
+                'a', b, dim=dim, weights=_weights, skipna=skipna_bool
+            )
         else:
             expected = accessor_func(
-                'a', 'b', dim, weights=_weights, skipna=skipna_bool
+                'a', 'b', dim=dim, weights=_weights, skipna=skipna_bool
             )
     assert_allclose(actual, expected)
