@@ -67,57 +67,6 @@ temporal_only_metrics = [
 ]
 
 
-@pytest.fixture
-def a():
-    times = pd.date_range('1/1/2000', '1/3/2000', freq='D')
-    lats = np.arange(4)
-    lons = np.arange(5)
-    data = np.random.rand(len(times), len(lats), len(lons))
-    return xr.DataArray(data, coords=[times, lats, lons], dims=['time', 'lat', 'lon'])
-
-
-@pytest.fixture
-def b(a):
-    b = a.copy()
-    b.values = np.random.rand(a.shape[0], a.shape[1], a.shape[2])
-    return b
-
-
-@pytest.fixture
-def b_nan(b):
-    return b.where(b < 0.5)
-
-
-@pytest.fixture
-def weights(a):
-    """Weighting array by cosine of the latitude."""
-    a_weighted = a.copy()
-    cos = np.abs(np.cos(a.lat))
-    data = np.tile(cos, (a.shape[0], a.shape[2], 1)).reshape(
-        a.shape[0], a.shape[1], a.shape[2]
-    )
-    a_weighted.values = data
-    return a_weighted
-
-
-@pytest.fixture
-def a_dask(a):
-    return a.chunk()
-
-
-@pytest.fixture
-def b_dask(b):
-    return b.chunk()
-
-
-@pytest.fixture
-def weights_dask(weights):
-    """
-    Weighting array by cosine of the latitude.
-    """
-    return weights.chunk()
-
-
 def adjust_weights(dim, weight_bool, weights):
     """
     Adjust the weights test data to only span the core dimension
@@ -153,7 +102,7 @@ def test_correlation_metrics_xr(a, b, dim, weight_bool, weights, metrics):
     # check that no chunks for no chunk inputs
     assert actual.chunks is None
 
-    dim, _ = _preprocess_dims(dim)
+    dim, _ = _preprocess_dims(dim, a)
     if len(dim) > 1:
         new_dim = '_'.join(dim)
         _a = a.stack(**{new_dim: dim})
@@ -182,33 +131,39 @@ def test_correlation_metrics_xr(a, b, dim, weight_bool, weights, metrics):
 @pytest.mark.parametrize('metrics', distance_metrics)
 @pytest.mark.parametrize('dim', AXES)
 @pytest.mark.parametrize('weight_bool', [True, False])
-def test_distance_metrics_xr(a, b, dim, weight_bool, weights, metrics):
+@pytest.mark.parametrize('skipna', [True, False])
+@pytest.mark.parametrize('has_nan', [True, False])
+def test_distance_metrics_xr(a, b, dim, weight_bool, weights, metrics, skipna, has_nan):
     """Test whether distance-based metric for xarray functions (from
      deterministic.py) give save numerical results as for numpy functions from
      np_deterministic.py)."""
     # unpack metrics
+    a = a.copy()
+    if has_nan:
+        a[0] = np.nan
+
     metric, _metric = metrics
     # Generates subsetted weights to pass in as arg to main function and for
     # the numpy testing.
     weights = adjust_weights(dim, weight_bool, weights)
     # median absolute error has no weights argument
     if metric is median_absolute_error:
-        actual = metric(a, b, dim)
+        actual = metric(a, b, dim, skipna=skipna)
     else:
-        actual = metric(a, b, dim, weights=weights)
+        actual = metric(a, b, dim, weights=weights, skipna=skipna)
     assert actual.chunks is None
 
-    dim, axis = _preprocess_dims(dim)
+    dim, axis = _preprocess_dims(dim, a)
     _a = a
     _b = b
     _weights = _preprocess_weights(_a, dim, dim, weights)
     axis = tuple(a.dims.index(d) for d in dim)
     if metric is median_absolute_error:
-        res = _metric(_a.values, _b.values, axis, skipna=False)
+        res = _metric(_a.values, _b.values, axis, skipna=skipna)
     else:
         # ensure _weights.values or None
         _weights = None if _weights is None else _weights.values
-        res = _metric(_a.values, _b.values, _weights, axis, skipna=False)
+        res = _metric(_a.values, _b.values, _weights, axis, skipna=skipna)
     expected = actual.copy()
     expected.values = res
     assert_allclose(actual, expected)
@@ -255,12 +210,18 @@ def test_correlation_metrics_xr_dask(
 @pytest.mark.parametrize('metrics', distance_metrics)
 @pytest.mark.parametrize('dim', AXES)
 @pytest.mark.parametrize('weight_bool', [True, False])
+@pytest.mark.parametrize('skipna', [True, False])
+@pytest.mark.parametrize('has_nan', [True, False])
 def test_distance_metrics_xr_dask(
-    a_dask, b_dask, dim, weight_bool, weights_dask, metrics
+    a_dask, b_dask, dim, weight_bool, weights_dask, metrics, skipna, has_nan
 ):
     """Test whether distance metrics for xarray functions can be lazy when
      chunked by using dask and give same results."""
     a = a_dask.copy()
+    if has_nan:
+        a = a.load()
+        a[0] = np.nan
+        a = a.chunk()
     b = b_dask.copy()
     weights = weights_dask.copy()
     # unpack metrics
@@ -271,15 +232,15 @@ def test_distance_metrics_xr_dask(
     if _weights is not None:
         _weights = _weights.load()
     if metric is median_absolute_error:
-        actual = metric(a, b, dim)
+        actual = metric(a, b, dim, skipna=skipna)
     else:
-        actual = metric(a, b, dim, weights=_weights)
+        actual = metric(a, b, dim, weights=_weights, skipna=skipna)
     # check that chunks for chunk inputs
     assert actual.chunks is not None
     if metric is median_absolute_error:
-        expected = metric(a.load(), b.load(), dim)
+        expected = metric(a.load(), b.load(), dim, skipna=skipna)
     else:
-        expected = metric(a.load(), b.load(), dim, weights=_weights)
+        expected = metric(a.load(), b.load(), dim, weights=_weights, skipna=skipna)
     assert expected.chunks is None
     assert_allclose(actual.compute(), expected)
 
@@ -287,7 +248,6 @@ def test_distance_metrics_xr_dask(
 @pytest.mark.parametrize('dim', AXES)
 @pytest.mark.parametrize('metric', [smape])
 def test_percentage_metric_in_interval_0_1(a, b, dim, metric):
-    print(a, b)
     """Test smape to be within bounds."""
     res = metric(a, b, dim)
     assert not (res < 0).any()
@@ -311,3 +271,65 @@ def test_pearson_r_integer():
     """Test whether arrays as integers work."""
     da = xr.DataArray([0, 1, 2], dims=['time'])
     assert pearson_r(da, da, dim='time') == 1
+
+
+@pytest.mark.parametrize('metrics', correlation_metrics + distance_metrics)
+@pytest.mark.parametrize('keep_attrs', [True, False])
+def test_keep_attrs(a, b, metrics, keep_attrs):
+    """Test keep_attrs for all metrics."""
+    metric, _metric = metrics
+    # ths tests only copying attrs from a
+    res = metric(a, b, 'time', keep_attrs=keep_attrs)
+    if keep_attrs:
+        assert res.attrs == a.attrs
+    else:
+        assert res.attrs == {}
+    da = xr.DataArray([0, 1, 2], dims=['time'])
+    assert pearson_r(da, da, dim='time') == 1
+
+
+@pytest.mark.parametrize('metrics', correlation_metrics + distance_metrics)
+def test_dim_None(a, b, metrics):
+    """Test that `dim=None` reduces all dimensions as xr.mean(dim=None) and fails for
+    effective metrics."""
+    metric, _metric = metrics
+    if metric in [effective_sample_size, spearman_r_eff_p_value, pearson_r_eff_p_value]:
+        with pytest.raises(ValueError) as excinfo:
+            metric(a, b, dim=None)
+        assert (
+            'Effective sample size should only be applied to a singular time dimension.'
+            in str(excinfo.value)
+        )
+    else:
+        metric, _metric = metrics
+        res = metric(a, b, dim=None)
+        assert len(res.dims) == 0, print(res.dims)
+
+
+@pytest.mark.parametrize('metrics', correlation_metrics + distance_metrics)
+def test_dim_empty_list(a, b, metrics):
+    """Test that `dim=[]` reduces no dimensions as xr.mean(dim=[]) and fails for
+    correlation metrics."""
+    if metrics in correlation_metrics:
+        metric, _metric = metrics
+        with pytest.raises(ValueError) as excinfo:
+            metric(a, b, dim=[])
+        assert 'requires `dim` not being empty, found dim' in str(excinfo.value)
+    elif metrics in distance_metrics:
+        metric, _metric = metrics
+        res = metric(a, b, dim=[])
+        assert len(res.dims) == len(a.dims), print(res.dims)
+
+
+@pytest.mark.parametrize('metrics', correlation_metrics + distance_metrics)
+def test_correlation_broadcasts(a, b, metrics):
+    """Test whether correlation metric broadcasts dimensions other than dim."""
+    # unpack metrics
+    metric, _metric = metrics
+    metric(a, b.isel(lat=0), dim='time')
+    metric(a, b.isel(lat=[0]), dim='time')
+    b_changed_coords = b.isel(lat=[0]).assign_coords(lat=[123])
+    print(metric.__name__)
+    with pytest.raises(ValueError) as e:
+        metric(a, b_changed_coords, dim='lat')
+    assert 'indexes along dimension' in str(e.value)
